@@ -38,11 +38,14 @@ import gtk
 import devede_other
 import gobject
 import cairo
+import dbus
+
 if (sys.platform=="win32") or (sys.platform=="win64"):
 	import win32api
 
 import devede_other
 import devede_video_convert
+import devede_ffmpeg_convert
 import devede_subtitles
 import devede_xml_menu
 import devede_delete
@@ -145,12 +148,46 @@ class create_all:
 		else:
 			return False,""
 
+	def check_mp3lame(self):
+		runner=devede_executor.executor()
+		runner.launch_program(["ffmpeg","-codecs"],sep_stderr=True,keep_out=True)
+		runner.wait_end2()
+		
+		print "Salida "+str(runner.cadena)
+		pos=runner.cadena.find("mp3lame")
+		if (pos==-1):
+			tree=devede_other.create_tree(self,"wno_mp3lame",self.gladefile)
+			w=tree.get_object("nomp3lame")
+			w.show()
+			w.run()
+			w.destroy()
+			self.has_mp3lame=False
+			return False
+		else:
+			self.has_mp3lame=True
+			return True
 	
 	def __init__(self,gladefile,structure,global_vars,callback):
 		
+		if (0!=devede_other.check_program("k3b -v")):
+			self.k3b_available=False
+		else:
+			self.k3b_available=True
+		
+		if (0!=devede_other.check_program("brasero --help")):
+			self.brasero_available=False
+		else:
+			self.brasero_available=True
+		
 		self.gladefile=gladefile
 		self.structure=structure
 		self.global_vars=global_vars
+		
+		self.has_mp3lame=True
+		if (global_vars["disctocreate"]=="divx") and (global_vars["use_ffmpeg"]):
+			if (False==self.check_mp3lame()):
+				return
+		
 		self.tree=devede_other.create_tree(self,"wprogress",self.gladefile)
 		
 		self.window=self.tree.get_object("wprogress")
@@ -203,23 +240,27 @@ class create_all:
 
 	def preview(self,filefolder):
 		
+		if (self.has_mp3lame==False):
+			self.window.destroy()
+			return
+		
 		self.init_queue()
 		newtree=devede_other.create_tree(self,"wpreview_dialog",self.gladefile,False)
 		timev=newtree.get_object("seconds_preview")
 		timev.set_value(60)
-		path=newtree.get_object("temporary_files")
-		path.set_filename(filefolder)
+		
 		w=newtree.get_object("wpreview_dialog")
 		w.show()
 		ret=w.run()
 		w.hide()
-		self.filefolder=path.get_filename()
+		self.filefolder=filefolder
 		if self.filefolder[-1]!=os.sep:
 			self.filefolder+=os.sep
 		self.seconds=timev.get_value()
 		w.destroy()
 		if ret!=-6:
-			return self.filefolder
+			self.window.destroy()
+			return
 
 		self.runner=None
 		self.queue.append(["PREVIEW"]) # Preview
@@ -233,7 +274,7 @@ class create_all:
 		except:
 			self.show_error(_("Failed to write to the destination directory.\nCheck that you have privileges and free space there."))
 			self.window.destroy()
-			return self.filefolder
+			return
 		
 		try:
 			os.remove(self.filefolder+"write_check")
@@ -244,7 +285,7 @@ class create_all:
 		self.erase_temp=True
 		self.timer=gobject.timeout_add(250,self.time_callback)
 		self.window.show()
-		return self.filefolder
+		return
 		
 
 	def on_iso_filename_activate(self,widg,ventana):
@@ -256,6 +297,12 @@ class create_all:
 		
 		self.time=0
 		
+		if (self.has_mp3lame==False):
+			self.window.hide()
+			self.window.destroy()
+			(self.main_window_callback)()
+			return False
+		
 		# first, check for empty titles
 		
 		empty=False
@@ -272,6 +319,9 @@ class create_all:
 			w.hide()
 			w.destroy()
 			if value!=-6:
+				self.window.hide()
+				self.window.destroy()
+				(self.main_window_callback)()
 				return False
 
 		# ask the folder and filename
@@ -282,6 +332,7 @@ class create_all:
 			wdir.set_current_folder(self.global_vars["finalfolder"])
 		wfile=newtree.get_object("iso_filename")
 		wfolder_dialog=newtree.get_object("wfolder_dialog")
+		do_shdw=newtree.get_object("do_shutdown")
 		wfile.set_text("movie")
 		wfile.connect("activate",self.on_iso_filename_activate,wfolder_dialog)
 		wfile.connect("changed",self.iso_changed,newtree)
@@ -290,18 +341,23 @@ class create_all:
 
 		wfolder_dialog.show()
 		wfile.grab_focus()
+		print "Entro en RUN"
 		value=wfolder_dialog.run()
-		
+		print "Salgo de RUN"
+		self.global_vars["shutdown_after_disc"]=do_shdw.get_active()
 		self.filename=wfile.get_text()
 		self.filename.replace("/","_")
 		self.filename.replace("|","_")
 		self.filename.replace("\\","_")
-			
+		
 		filefolder=wdir.get_current_folder()
 		
 		wfolder_dialog.hide()
 		wfolder_dialog.destroy()
 		if value!=-6:
+			self.window.hide()
+			self.window.destroy()
+			(self.main_window_callback)()
 			return False
 		
 		self.global_vars["finalfolder"]=filefolder
@@ -323,6 +379,9 @@ class create_all:
 			w.hide()
 			w.destroy()
 			if value!=2:
+				self.window.hide()
+				self.window.destroy()
+				(self.main_window_callback)()
 				return False
 		
 		try:
@@ -433,13 +492,19 @@ class create_all:
 			else:
 				encpass = int(action[0][1])
 			print "Segundos "+str(self.seconds)
-			self.runner=devede_video_convert.video_converter(self.global_vars,self.structure[title][chapter+1],self.filename,self.filefolder,self.partial,self.label,self.global_vars["disctocreate"],title+1,chapter+1,self.global_vars["multicore"],self.seconds, encpass)
+			if (self.global_vars["use_ffmpeg"]):
+				self.runner=devede_ffmpeg_convert.video_converter_ffmpeg(self.global_vars,self.structure[title][chapter+1],self.filename,self.filefolder,self.partial,self.label,self.global_vars["disctocreate"],title+1,chapter+1,self.global_vars["multicore"],self.seconds, encpass,self.global_vars["AC3_fix"])
+			else:
+				self.runner=devede_video_convert.video_converter(self.global_vars,self.structure[title][chapter+1],self.filename,self.filefolder,self.partial,self.label,self.global_vars["disctocreate"],title+1,chapter+1,self.global_vars["multicore"],self.seconds, encpass,self.global_vars["AC3_fix"])
 			return True
 		
 		if action[0]=="C2":
 			title=action[1]
 			chapter=action[2]
-			self.runner=devede_video_convert.video_converter(self.structure[title][chapter+1],self.filename,self.filefolder,self.partial,self.label,self.global_vars["disctocreate"],title+1,chapter+1,self.global_vars["multicore"],self.seconds, 2)
+			if (self.global_vars["use_ffmpeg"]):
+				self.runner=devede_ffmpeg_convert.video_converter_ffmpeg(self.structure[title][chapter+1],self.filename,self.filefolder,self.partial,self.label,self.global_vars["disctocreate"],title+1,chapter+1,self.global_vars["multicore"],self.seconds, 2,self.global_vars["AC3_fix"])
+			else:
+				self.runner=devede_video_convert.video_converter(self.structure[title][chapter+1],self.filename,self.filefolder,self.partial,self.label,self.global_vars["disctocreate"],title+1,chapter+1,self.global_vars["multicore"],self.seconds, 2,self.global_vars["AC3_fix"])
 			return True
 		
 		if action[0]=="S":
@@ -473,7 +538,7 @@ class create_all:
 				w.hide()
 				if ret!=-6:
 					break
-				while gtk.events_pending():
+				while gtk.events_pending():	
 					gtk.main_iteration()
 			w.destroy()
 			os.remove(fname)
@@ -503,11 +568,44 @@ class create_all:
 
 		self.window.hide()
 		self.window.destroy()
+		if (self.global_vars["shutdown_after_disc"]):
+			print "\n\nApago el ordenador\n\n"
+			failure=False
+			
+			# First, try with ConsoleKit
+			try:
+				bus = dbus.SystemBus()
+				bus_object = bus.get_object("org.freedesktop.ConsoleKit", "/org/freedesktop/ConsoleKit/Manager")
+				bus_object.Stop(dbus_interface="org.freedesktop.ConsoleKit.Manager")
+			except:
+				failure=True
+			if (failure):
+				failure=False
+				
+				# If it fails, try with HAL
+				try:
+					bus = dbus.SystemBus()
+					bus_object = bus.get_object("org.freedesktop.Hal", "/org/freedesktop/Hal/devices/computer")
+					bus_object.Shutdown(dbus_interface="org.freedesktop.Hal.Device.SystemPowerManagement")
+				except:
+					failure=True
+
+			if (failure==False):
+				gtk.main_quit()
+
 		newtree=devede_other.create_tree(self,"wend_dialog",self.gladefile,False)
 		label=newtree.get_object("elapsed")
 		tiempo2=devede_other.return_time(time.time()-self.tiempo,True)
 		label.set_text(tiempo2)
 		window=newtree.get_object("wend_dialog")
+		burn = newtree.get_object("burn_button")
+		# only enable button if k3b is available
+		# TODO: support other burners
+		
+		if (self.k3b_available==False) and (self.brasero_available==False):
+			burn.set_sensitive(False)
+		
+		burn.connect('clicked', self.burn)
 		window.show()
 		window.run()
 		window.hide()
@@ -517,6 +615,24 @@ class create_all:
 		gc.collect()
 		(self.main_window_callback)()
 
+	def burn(self, widget):
+
+		"""Burns resulting iso"""
+
+		path = self.filefolder + self.filename + ".iso"
+		print path
+		parameters = []
+		if (self.brasero_available):
+			parameters.append("brasero")
+			parameters.append("--image="+path)
+		else:
+			parameters.append("k3b")
+			parameters.append("--burn")
+			parameters.append(path)
+		runner=devede_executor.executor()
+		runner.launch_program(parameters,output=False)
+		runner.wait_end()
+
 
 	def show_error(self,message):
 		
