@@ -36,6 +36,7 @@ import devede_executor
 import devede_convert
 import devede_dialogs
 import devede_help
+import math
 
 # How to add new file options:
 
@@ -153,7 +154,7 @@ class file_get_params(devede_executor.executor):
 				self.audio=audio
 			return False,audio
 		
-		if aspect_ratio==0.0:
+		if aspect_ratio==0.0 or math.isnan(aspect_ratio) :
 			aspect_ratio=(float(width))/(float(height))
 			if aspect_ratio<=1.5:
 				aspect_ratio=(4.0/3.0)
@@ -392,25 +393,32 @@ class newfile(file_get_params):
 		else:
 			length=7
 		
+		cadena2=""
 		for elemento in data:
-			if (elemento=="\n") and (item!=""):
-				if item[:8]=="file:///":
-					lista.append(item[length:])
-					item=""
-					continue
-			if ord(elemento)<32:
-				continue
-			if mode==0:
-				if elemento=="%":
-					mode=1
-				else:
-					item+=elemento
-			elif mode==1:
-				tempo=elemento
-				mode=2
+			if (ord(elemento)<32):
+				cadena2+="\n"
 			else:
-				mode=0
-				item+=chr(int(tempo+elemento,16))
+				cadena2+=elemento
+		
+		while(True):
+			pos=cadena2.find("file:///")
+			if (pos==-1):
+				break
+			pos2=cadena2.find("\n",pos)
+			if (pos2==-1):
+				cadena=cadena2[pos+length:]
+				cadena2=""
+			else:
+				cadena=cadena2[pos+length:pos2]
+				cadena2=cadena2[pos2:]
+			while (True):
+				pos=cadena.find("%")
+				if (pos==-1):
+					break
+				cadena=cadena[:pos]+chr(int(cadena[pos+1:pos+3],16))+cadena[pos+3:]
+			lista.append(cadena)
+		
+		
 		print "Dragged files: "+str(lista)
 		return lista
 
@@ -423,6 +431,19 @@ class file_properties(newfile):
 	""" This class manages the properties window, where the user can choose the properties
 		of each video file """
 	
+	def motion_cb(self,wid, context, x, y, time):
+		context.drag_status(gtk.gdk.ACTION_COPY, time)
+		return True
+	
+	def drop_cb(self, wid, context, x, y, time):
+		# Used with windows drag and drop
+		print 'drop'
+		self.have_drag = False
+		if context.targets:
+			wid.drag_get_data(context, context.targets[0], time)
+			return True
+		return False
+	
 	# public methods	
 	
 	def __init__(self,global_vars,title,chapter,structure,callback_refresh):
@@ -438,7 +459,10 @@ class file_properties(newfile):
 		self.tree=devede_other.create_tree(self,"wfile",self.gladefile)
 		self.window=self.tree.get_object("wfile")
 		self.window.show()
-		self.window.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT | gtk.DEST_DEFAULT_DROP,[ ( "text/plain", 0, 80 ), ( "video/*", 0, 81) ],gtk.gdk.ACTION_COPY)
+		self.window.drag_dest_set(0,[],0)
+		self.window.connect('drag_drop', self.drop_cb)
+		self.window.connect('drag_motion', self.motion_cb)
+
 		w = self.tree.get_object("expander_advanced")
 		w.set_expanded(self.global_vars["expand_advanced"])
 		
@@ -568,7 +592,7 @@ class file_properties(newfile):
 
 	def custom_video_filter(self, filter_info=None, data=None):
 		"""Custom file filter.  Filter for video files when running on win32"""
-		video_types=["wmv", "avi", "asf", "flv", "bin", "vob", "es", "ps", "pes","qt", "mov", "mp4", "mpg", "mpeg", "rm", "mkv", "nut", "nsv", "vivo", "fli", "yuv4mpeg", "cpk","ogm", "asx", "3gp"]
+		video_types=["m2ts","wmv", "avi", "asf", "flv", "bin", "vob", "es", "ps", "pes","qt", "mov", "mp4", "mpg", "mpeg", "rm", "mkv", "nut", "nsv", "vivo", "fli", "yuv4mpeg", "cpk","ogm", "asx", "3gp"]
 		if os.path.splitext(filter_info[2])[1][1:] in video_types:
 			return True
 		return False
@@ -988,8 +1012,7 @@ class file_properties(newfile):
 		self.global_vars["number_actions"]=1
 		tmp_structure=[["",self.file_properties]]
 		converter=devede_convert.create_all(self.gladefile,tmp_structure,self.global_vars,self.callback)
-		self.global_vars["temp_folder"]=converter.preview(self.global_vars["temp_folder"])
-		print "Temp folder changed to "+str(self.global_vars["temp_folder"])
+		converter.preview(self.global_vars["temp_folder"])
 
 
 	def on_file_help_clicked(self,widget):
@@ -1076,7 +1099,7 @@ class file_properties(newfile):
 	def set_widgets(self):
 		
 		""" sets the widgets to the values specificated in SELF.FILE_PROPERTIES """
-
+		
 		dsize,minvid,maxvid=devede_other.get_dvd_size(None,self.disctocreate)
 
 		if self.disctocreate=="vcd":
@@ -1155,6 +1178,8 @@ class file_properties(newfile):
 		w=self.tree.get_object("turbo1stpass")
 		w.set_active(self.file_properties["turbo1stpass"])
 		
+		w.set_sensitive(self.file_properties["twopass"])
+		
 		if self.file_properties["mbd"]==0:
 			w=self.tree.get_object("mbd")
 		elif self.file_properties["mbd"]==1:
@@ -1281,6 +1306,24 @@ class file_properties(newfile):
 			w.show()
 
 		self.refresh_subtitles()
+		
+		if (self.global_vars["use_ffmpeg"]):
+			use_ffmpeg=False
+		else:
+			use_ffmpeg=True
+		
+		self.tree.get_object("turbo1stpass").set_visible(use_ffmpeg)
+		self.tree.get_object("deinterlace_md").set_visible(use_ffmpeg)
+		self.tree.get_object("deinterlace_l5").set_visible(use_ffmpeg)
+		self.tree.get_object("deinterlace_lb").set_visible(use_ffmpeg)
+		self.tree.get_object("lavcopts_label").set_visible(use_ffmpeg)
+		self.tree.get_object("lameopts_label").set_visible(use_ffmpeg)
+		self.tree.get_object("custom_params_lavcopts").set_visible(use_ffmpeg)
+		self.tree.get_object("custom_params_lameopts").set_visible(use_ffmpeg)
+		#self.tree.get_object("volume_frame").set_visible(use_ffmpeg)
+		self.tree.get_object("audiotrack_box").set_visible(use_ffmpeg)
+		self.tree.get_object("frame_field_order").set_visible(use_ffmpeg)
+		
 
 	def refresh_subtitles(self):
 
@@ -1485,8 +1528,9 @@ class file_properties(newfile):
 		w.set_sensitive(grupo2 and isvob)
 		w=self.tree.get_object("twopass")
 		w.set_sensitive(grupo2 and isvob)
+		tp = w.get_active()
 		w=self.tree.get_object("turbo1stpass")
-		w.set_sensitive(grupo2 and isvob)
+		w.set_sensitive(grupo2 and isvob and tp)
 		w=self.tree.get_object("deinterlace")
 		w.set_sensitive(grupo2 and isvob)
 		w=self.tree.get_object("deinterlace_lb")
