@@ -34,10 +34,16 @@ class xml_files(devede_executor.executor):
 	def __init__(self,pbar,filename,filefolder,structure,global_vars,proglabel,extcr=None):
 	
 		devede_executor.executor.__init__(self,filename,filefolder,pbar)
+		
+		self.height2=0.05
+		self.height=0.0391604010025
+		
 		self.proglabel=proglabel
 		self.print_error=_("Failed to create the menues.")
 		self.output=False
 		self.structure=structure
+		self.AC3_fix=global_vars["AC3_fix"]
+		self.use_ffmpeg=global_vars["use_ffmpeg"]
 		self.with_menu=global_vars["with_menu"]
 		if (len(structure)==1) and (len(structure[0])==2) and (not self.with_menu):
 			self.onlyone=True
@@ -67,10 +73,20 @@ class xml_files(devede_executor.executor):
 
 		self.align=global_vars["menu_alignment"]
 		self.halign=global_vars["menu_halignment"]
+		self.radius=0.0375
+
+		self.top_margin=0.75*global_vars["menu_top_margin"]
+		self.bottom_margin=0.75*global_vars["menu_bottom_margin"]
+		self.left_margin=global_vars["menu_left_margin"]
+		self.right_margin=global_vars["menu_right_margin"]
 		self.extcr=extcr
 		
-		self.elements_per_menu=10
-		self.lines_per_menu=10
+		self.elements_per_menu=int((0.75-(self.top_margin+self.bottom_margin))/self.height2)
+		self.lines_per_menu=self.elements_per_menu
+		print "Elementos por menu: "+str(self.elements_per_menu)
+		
+		if (len(self.structure))>self.elements_per_menu:
+			self.elements_per_menu-=1
 		
 		counter=0
 		if self.with_menu:
@@ -509,7 +525,7 @@ class xml_files(devede_executor.executor):
 
 		cantidad=len(self.structure[first_element:first_element+self.elements_per_menu])
 		if self.align!=0:
-			offset=(self.lines_per_menu-cantidad)/self.align
+			offset=(self.elements_per_menu-cantidad)/self.align
 		else:
 			offset=0
 			
@@ -524,10 +540,27 @@ class xml_files(devede_executor.executor):
 			fichero.write(' image="'+self.expand_xml(self.filefolder+self.filename)+'_menu'+str(nelement)+'_bg_inactive_out.png"')
 			fichero.write(' highlight="'+self.expand_xml(self.filefolder+self.filename)+'_menu'+str(nelement)+'_bg_active_out.png"')
 			fichero.write(' select="'+self.expand_xml(self.filefolder+self.filename)+'_menu'+str(nelement)+'_bg_select_out.png" >\n')
-			if self.menu_PAL:
-				coord_y=[(92,128),(130,166),(168,204),(206,242),(244,280),(282,320),(322,358),(360,396),(398,436),(438,474),(476,512)]
+			
+			counter=first_element
+			if (self.align==0):
+				pos_y=self.top_margin
+			elif self.align==1:
+				pos_y=0.75-self.bottom_margin-self.height2*float(len(self.structure[counter:counter+self.lines_per_menu]))
 			else:
-				coord_y=[(76,106),(108,138),(140,170),(172,202),(204,234),(236,266),(268,298),(300,330),(332,362),(364,394),(396,426)]
+				pos_y=(self.top_margin+(0.75-self.bottom_margin)-self.height2*float(len(self.structure[counter:counter+self.lines_per_menu])))/2
+			pos_y3=0.75-self.bottom_margin-self.height2#pos_y+self.height2*float((len(self.structure[counter:counter+self.lines_per_menu])))
+			pos_y/=0.75
+			pos_y3/=0.75
+			inc_y=self.height2/0.75
+			
+			if self.menu_PAL:
+				pos_y*=576.0
+				pos_y3*=576.0
+				inc_y*=576.0
+			else:
+				pos_y*=480.0
+				pos_y3*=480.0
+				inc_y*=480.0
 			
 			if (nelement!=0):
 				has_previous=True
@@ -541,7 +574,8 @@ class xml_files(devede_executor.executor):
 			
 			for contador in range(cantidad):
 				fichero.write('<button name="boton'+str(nelement)+"x"+str(contador))
-				fichero.write('" x0="0" y0="'+str(coord_y[contador+offset][0])+'" x1="719" y1="'+str(coord_y[contador+offset][1])+'"')
+				fichero.write('" x0="0" y0="'+str(int(pos_y))+'" x1="719" y1="'+str(int(pos_y+inc_y-2))+'"')
+				pos_y+=inc_y
 				if contador!=0:
 					fichero.write(' up="boton'+str(nelement)+"x")
 					fichero.write(str(contador-1))
@@ -560,10 +594,10 @@ class xml_files(devede_executor.executor):
 				if has_previous:
 					fichero.write(' left="boton'+str(nelement)+'p"')
 				fichero.write(' > </button>\n')
-				
+			
 			if has_previous:
 				fichero.write('<button name="boton'+str(nelement)+'p"')
-				fichero.write(' x0="0" y0="'+str(coord_y[10][0])+'" x1="359" y1="'+str(coord_y[10][1])+'"')
+				fichero.write(' x0="0" y0="'+str(int(pos_y3))+'" x1="'+str(int((self.left_margin+self.right_margin)/2))+'" y1="'+str(int(pos_y3+inc_y-2))+'"')
 				fichero.write(' up="boton'+str(nelement)+'x'+str(cantidad-1)+'"')
 				if has_next:
 					fichero.write(' right="boton'+str(nelement)+'n"')
@@ -571,7 +605,7 @@ class xml_files(devede_executor.executor):
 
 			if has_next:
 				fichero.write('<button name="boton'+str(nelement)+'n"')
-				fichero.write(' x0="360" y0="'+str(coord_y[10][0])+'" x1="719" y1="'+str(coord_y[10][1])+'"')
+				fichero.write(' x0="'+str(int((self.left_margin+2+self.right_margin)/2))+'" y0="'+str(int(pos_y3))+'" x1="719" y1="'+str(int(pos_y3+inc_y-2))+'"')
 				fichero.write(' up="boton'+str(nelement)+'x'+str(cantidad-1)+'"')
 				if has_previous:
 					fichero.write(' left="boton'+str(nelement)+'p"')
@@ -588,30 +622,31 @@ class xml_files(devede_executor.executor):
 
 		""" paints the rounded rectangles used as background for titles and buttons """
 
-		radius=0.0375
+		radius=self.radius
 		border=0.0048
 		linea=0.0024
 		# I created the button image for this size, so I must respect it :(
 		#xb,yb,width,height,cx,cy=cr.text_extents("Título 1")
 
-		height=0.0391604010025
+		height=self.height
 
 		if width==1:
 			half_button=False
 		else:
 			half_button=True
-		width-=2*self.margin_x
+		
+		width-=self.left_margin+self.right_margin#2*self.margin_x
 		
 		if half_button: # we want half button
 			if x==0:
-				xi=x+self.margin_x
-				xf=xi+0.5-self.margin_x*1.5
+				xi=self.left_margin+radius
+				xf=((self.left_margin+1-self.right_margin)/2.0)-radius
 			else:
-				xf=x-self.margin_x
-				xi=xf-0.5+self.margin_x*1.5
+				xi=((self.left_margin+1-self.right_margin)/2.0)+radius
+				xf=1-self.right_margin-radius
 		else:
-			xi=x+self.margin_x
-			xf=xi+width
+			xi=self.left_margin+radius
+			xf=1-self.right_margin-radius
 
 		cr.set_line_width(linea)
 	
@@ -635,8 +670,7 @@ class xml_files(devede_executor.executor):
 		else:
 			s=False
 			arrowx-=0.0175
-			
-			
+
 		self.menu_paint_arrow(cr, arrowx+self.shadow_offset, y+self.shadow_offset, s, height, shcolor)
 		self.menu_paint_arrow(cr, arrowx, y, s, height, fgcolor)
 
@@ -670,7 +704,7 @@ class xml_files(devede_executor.executor):
 
 		# I created the button image for this size, so I must respect it :(
 		#xb,yb,width,height,cx,cy=cr.text_extents("Título 1")
-		height=0.0391604010025
+		height=self.height
 	
 		cr.set_font_size(fontsize2)
 		xb,y2,width,h2,cx,cy2=cr.text_extents(texto)
@@ -691,11 +725,11 @@ class xml_files(devede_executor.executor):
 		coa=float(fontcolor[3])/65535.0
 		cr.set_source_rgba(cor,cog,cob,coa)
 		if halign==2: # center
-			nx=.5-width/2.0-xb
+			nx=((self.left_margin+1-self.right_margin)-width)/2
 		elif halign==0: # left
-			nx=self.margin_x
+			nx=self.left_margin+self.radius
 		elif halign==1: # right
-			nx=1.0-self.margin_x-width
+			nx=1.0-self.right_margin-width-self.radius
 		
 		nx+=x
 		
@@ -708,7 +742,7 @@ class xml_files(devede_executor.executor):
 		cr.show_text(texto)
 
 
-	def create_menu_bg(self,counter,element,paint_bg=0):
+	def create_menu_bg(self,counter,element,paint_bg=0,preview=False):
 
 		""" Paints the menu in a Cairo surface.	PAINT_BG can be:
 			0: paints everything for the base picture
@@ -764,11 +798,15 @@ class xml_files(devede_executor.executor):
 
 		cr.scale(sf.get_width(),1.33*sf.get_height()) # picture gets from 0 to 1 in X and from 0 to 0.75 in Y
 
-		pos_y=0.125
+		if (self.align==0):
+			pos_y=self.top_margin
+		elif self.align==1:
+			pos_y=0.75-self.bottom_margin-self.height2*float(len(self.structure[counter:counter+self.lines_per_menu]))
+		else:
+			pos_y=(self.top_margin+(0.75-self.bottom_margin)-self.height2*float(len(self.structure[counter:counter+self.lines_per_menu])))/2
+		pos_y3=0.75-self.bottom_margin-self.height2
+		#pos_y3=pos_y+self.height2*float((self.lines_per_menu)-1)
 		
-		if self.align!=0:
-			pos_y+=0.05*float((self.lines_per_menu-len(self.structure[counter:counter+self.elements_per_menu]))/self.align)
-
 		fontname,fontstyle,fontslant,fontsize=devede_other.get_font_params(self.font_name)
 
 		shadowcolor=self.shadowcolor
@@ -787,9 +825,9 @@ class xml_files(devede_executor.executor):
 				arrowshadowcolor=None
 			elif paint_bg==2:
 				bgcolor=[0,0,0,0]
-				fontcolor=self.activecolor
-				arrowcolor=self.activecolor
-				strokecolor=self.activecolor
+				fontcolor=self.fontcolor
+				arrowcolor=self.fontcolor
+				strokecolor=self.fontcolor
 				arrowshadowcolor=None
 			else:
 				bgcolor=[0,0,0,0]
@@ -804,7 +842,7 @@ class xml_files(devede_executor.executor):
 				if paint_bg==0: # print the shadow first
 					self.menu_set_text(cr,self.shadow_offset,pos_y+self.shadow_offset,self.halign,entrada[0]["nombre"],bgcolor,shadowcolor,fontname,fontstyle,fontslant,fontsize)
 				self.menu_set_text(cr,0,pos_y,self.halign,entrada[0]["nombre"],bgcolor,fontcolor,fontname,fontstyle,fontslant,fontsize)
-				pos_y+=0.05
+				pos_y+=self.height2
 
 			print "Paint_bg "+str(paint_bg)+" title text: "+str(self.title_text)
 			if (paint_bg==0) and (self.title_text!=""):
@@ -813,13 +851,12 @@ class xml_files(devede_executor.executor):
 				self.menu_set_text(cr,self.shadow_offset, 0.075+self.shadow_offset,2,self.title_text, [0,0,0,0], self.title_shadow, fontname, fontstyle, fontslant, fontsize, False)
 				self.menu_set_text(cr,0, 0.075,2,self.title_text, [0,0,0,0], self.title_color, fontname, fontstyle, fontslant, fontsize, False)
 
-			pos_y=0.625
 			# check if we have to paint the NEXT MENU button
 			if (element!=self.nmenues-1) and (self.nmenues>1):
-				self.menu_set_bg(cr, 1, pos_y, 0.4, bgcolor,arrowcolor,arrowshadowcolor)
+				self.menu_set_bg(cr, 1, pos_y3, 0.4, bgcolor,arrowcolor,arrowshadowcolor)
 			# check if we have to paint the PREVIOUS MENU button
 			if (element!=0) or ((self.extcr!=None) and (self.nmenues>1)):
-				self.menu_set_bg(cr, 0, pos_y, 0.4, bgcolor,arrowcolor,arrowshadowcolor)
+				self.menu_set_bg(cr, 0, pos_y3, 0.4, bgcolor,arrowcolor,arrowshadowcolor)
 
 		if self.extcr==None:
 			if paint_bg==0:
@@ -833,10 +870,71 @@ class xml_files(devede_executor.executor):
 		return sf
 
 	
+	def create_menu_mpg_ffmpeg(self,counter):
+
+		self.mplexed=False
+		command_var=[]
+		command_var.append("ffmpeg")
+	
+		currentfile=self.filefolder+self.filename+"_menu_"+str(counter)+".mpg"
+	
+		audio=self.menu_sound
+		command_var.append("-loop_input")
+		
+		command_var.append("-f")
+		command_var.append("image2")
+		command_var.append("-t")
+		command_var.append(str(1+self.menu_sound_duration))
+		command_var.append("-i")
+		command_var.append(self.filefolder+self.filename+"_menu"+str(counter)+"_bg.png")
+		command_var.append("-i")
+		command_var.append(audio)
+	
+		command_var.append("-y")
+		command_var.append("-target")
+		if self.menu_PAL:
+			command_var.append("pal-dvd")
+		else:
+			command_var.append("ntsc-dvd")
+		command_var.append("-s")
+		if self.menu_PAL:
+			command_var.append("720x576")
+		else:
+			command_var.append("720x480")
+		command_var.append("-g")
+		command_var.append("12")
+		command_var.append("-ac")
+		command_var.append("2")
+		command_var.append("-trellis")
+		command_var.append("1")
+		command_var.append("-mbd")
+		command_var.append("2")
+		command_var.append("-b")
+		command_var.append("1000000")
+		command_var.append("-ab")
+		command_var.append("128000")
+		command_var.append("-aspect")
+		command_var.append("4:3")
+
+
+		#audio="/home/raster/Escritorio/lazy.mp3"
+
+		
+		command_var.append(currentfile)
+		origDir=os.getcwd()
+		print "Lanzo "+str(command_var)
+
+		self.launch_program(command_var)
+	
+	
 	def create_menu_mpg(self,counter):
 	
 		print "Creating menus"
-		
+	
+		if (self.use_ffmpeg):
+			self.create_menu_mpg_ffmpeg(counter)
+			return
+	
 		self.mplexed=False
 		command_var=[]
 		if (sys.platform=="win32") or (sys.platform=="win64"):
@@ -884,7 +982,12 @@ class xml_files(devede_executor.executor):
 			lavcopts=""
 		lavcopts+="vcodec=mpeg2video:sc_threshold=1000000000:cgop:trell:mbd=2:vstrict=0:"
 		lavcopts+="vrc_maxrate=7000:vrc_buf_size=1835:vbitrate=1000:keyint=12:"
-		lavcopts+="acodec=ac3:abitrate=128:aspect="+wide
+		lavcopts+="acodec="
+		if self.AC3_fix:
+			lavcopts+="ac3_fixed"
+		else:
+			lavcopts+="ac3"
+		lavcopts+=":abitrate=128:aspect="+wide
 		command_var.append(lavcopts)
 		command_var.append("-o")
 		command_var.append(currentfile)
